# -*- coding: utf-8 -*-

class Emitter:

    """Event emitter mixin class."""

    def add_events(self, *event_names):
        """Add event names which may be registered by user.

        This should be called only by subclasses.
        This serves also as initializer, other methods of Emitter
        will not work if add_events was not called.

        """
        if not hasattr(self, '_event_handlers'):
            self._event_handlers = dict()
        for event_name in event_names:
            self._event_handlers[event_name] = []

    def connect(self, event_name, handler):
        """Connect event handler to event name.

        Add handler to the end of handler list.

        """
        if event_name in self._event_handlers:
            self._event_handlers[event_name].append(handler)
        else:
            raise KeyError('Unknown event: %s', event_name)

    def disconnect(self, event_name, handler=None):
        """Remove event handler from the list.

        If no handler is given, remove all handlers.

        """
        if event_name in self._event_handlers:
            if handler:
                self._event_handlers[event_name].remove(handler)
            else:
                self._event_handlers[event_name][:] = []
        else:
            raise KeyError('Unknown event: %s', event_name)

    def is_connected(self, event_name):
        """Test if any handlers are connected to event name.

        Return True if event handler list is not empty,
        False otherwise.

        """
        if event_name in self._event_handlers:
            return bool(self._event_handlers[event_name])
        else:
            raise KeyError('Unknown event: %s', event_name)

    def handle(self, event_name, *args, **kwargs):
        """Emit the event.

        Call all handlers from event's handler list,
        starting from first added handler.

        Return True when one of the handlers returns True,
        False otherwise.

        """
        handled = False
        for handler in self._event_handlers[event_name]:
            res = handler(*args, **kwargs)
            if res:
                handled = True
        return handled

    def emit(self, event, *args, **kwargs):
        """Emit event.

        This is used by original event source when the event is detected.

        """
        try:
            getattr(self, event)(*args, **kwargs)
        except AttributeError:
            self.handle(event, *args, **kwargs)

